"""
health controller.
"""
from marshmallow import Schema, fields
from microcosm_flask.conventions.base import Convention
from microcosm_flask.conventions.encoding import dump_response_data
from microcosm_flask.conventions.registry import response
from microcosm_flask.namespaces import Namespace
from microcosm_flask.operations import Operation


class PingSchema(Schema):
    status = fields.String()
    name = fields.String()
    ok = fields.Bool()


class SagemakerHealthConvention(Convention):
    def __init__(self, graph):
        super(SagemakerHealthConvention, self).__init__(graph)

    def configure_query(self, ns, definition):
        @self.add_route("/api/health", Operation.Query, ns)
        # Stephens:
        #
        # We need /api/health for AI projects to run on ECS
        # This is a temp solution untill we complete the migration
        #
        # Currently health check exist on /health but not /api/health
        # because we removed /api from route prefix
        #
        # see commands/config.py `build_route_path`
        #
        @response(PingSchema())
        def health_check_query():
            response_data = dict(
                name=self.graph.metadata.name,
                ok=True,
            )
            return dump_response_data(PingSchema(), response_data, 200)


def configure_health(graph):
    ns = Namespace(
        subject="/api/health",
        version=None
    )
    convention = SagemakerHealthConvention(graph)
    convention.configure(ns, query=tuple())



"""
Example CRUD routes tests.

Tests are sunny day cases under the assumption that framework conventions
handle most error conditions.

"""
from hamcrest import (
    assert_that,
    equal_to,
    has_entries,
    is_,
)
from hamcrest.core.base_matcher import BaseMatcher

from microcosm_sagemaker.testing.route import RouteTestCase


class InvocationsRouteTestCase(RouteTestCase):
    """
    Helper base class for writing tests of the invocations route.

    """

    request_json: dict
    response_items_matcher: BaseMatcher

    def test_search(self) -> None:
        """
        Invokes the invocations endpoint with `request_json`, and checks the
        `items` entry of the response against `response_items_matcher`.

        """
        uri = "/invocations"

        response = self.client.post(
            uri,
            json=self.request_json,
        )

        assert_that(response.status_code, is_(equal_to(200)))
        assert_that(
            response.json,
            has_entries(
                items=self.response_items_matcher,
            ),
        )



from pathlib import Path
from typing import Union


class InputData:
    def __init__(self, path: Union[str, Path]):
        self.path = Path(path)

    def __repr__(self) -> str:
        return f'{type(self).__name__}("{str(self.path)}")'
